/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.book;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import fr.aliasource.utils.RunnableExtensionLoader;
import fr.aliasource.webmail.book.AbstractContactGroup.Id;
import fr.aliasource.webmail.book.AbstractContactGroup.Origin;
import fr.aliasource.webmail.book.impl.AllContacts;
import fr.aliasource.webmail.common.IAccount;
import fr.aliasource.webmail.common.LoginUtils;
import fr.aliasource.webmail.proxy.ProxyConfiguration;

/**
 * Address book sources management
 */
public class BookManager {

	public static final Origin ALL_SOURCE_ORIGIN = new Origin("agregate");
	public static final Id ALL_SOURCE_ID = new Id("all", ALL_SOURCE_ORIGIN);

	private Log logger;

	private Set<IBookSource> sources;
	private Set<IAddressCollector> collectors;

	private ProxyConfiguration proxyConfiguration;

	BookManager(ProxyConfiguration proxyConfiguration) {
		this.logger = LogFactory.getLog(getClass());
		this.proxyConfiguration = proxyConfiguration;

		registerSources();
		registerCollectors();

		logger.info("AddressBook Manager created with " + sources.size()
				+ " source(s)");
	}

	private final void registerSources() {
		List<IBookSource> plugins = getPluginSources();
		Set<IBookSource> sources = Sets.newHashSet(plugins);
		IBookSource agregateSource = new AllContacts(plugins);
		sources.add(agregateSource);
		for (IBookSource source: sources) {
			source.init(proxyConfiguration);
		}
		this.sources = sources;
	}

	private void registerCollectors() {
		List<IAddressCollector> pluginCollectors = getPluginCollectors();
		for (IAddressCollector collector: pluginCollectors) {
			collector.init(proxyConfiguration);
		}
		this.collectors = Sets.newHashSet(pluginCollectors);
	}
	
	private List<IBookSource> getPluginSources() {
		RunnableExtensionLoader<IBookSource> rel = new RunnableExtensionLoader<IBookSource>();
		List<IBookSource> bs = rel.loadExtensions(BookActivator.PLUGIN_ID,
				"booksource", "book_source", "implementation");
		return bs;
	}

	private List<IAddressCollector> getPluginCollectors() {
		RunnableExtensionLoader<IAddressCollector> rel = new RunnableExtensionLoader<IAddressCollector>();
		return rel.loadExtensions(BookActivator.PLUGIN_ID, "addresscollector", "address_collector", "implementation");
	}
	
	void init() {
	}

	public List<ContactGroup> getGroups(String userId, String userPassword) {
		List<ContactGroup> ret = new ArrayList<ContactGroup>();
		for (IBookSource source: sources) {
			List<? extends ContactGroup> groups = source.listGroups(userId, userPassword);
			ret.addAll(groups);
		}
		return ret;
	}

	public ContactGroup getGroupFromId(String userId, String userPassword, fr.aliasource.webmail.book.ContactGroup.Id id) {
		for (ContactGroup group: getGroups(userId, userPassword)) {
			if (group.getId().equals(id)) {
				return group;
			}
		}
		return null;
	}

	
	public ContactGroup getGroupFromString(String userId, String userPassword, String origin, String id) {
		for (ContactGroup group: getGroups(userId, userPassword)) {
			if (group.getId().originAsString().equals(origin) && group.getId().idAsString().equals(id)) {
				return group;
			}
		}
		return null;
	}

	void shutdown() {
		for (IBookSource a : sources) {
			a.shutdown();
		}
	}

	public void collectContacts(IAccount account, List<MinigContact> contacts) throws AddressCollectException {
		String userId = LoginUtils.lat(account);
		String userPassword = account.getUserPassword();
		ContactGroup agregateGroup = getGroupFromId(userId, userPassword, ALL_SOURCE_ID);

		List<MinigContact> toInsert = Lists.newArrayListWithCapacity(contacts.size());
		for (MinigContact c : contacts) {
			if (checkContactExistsInGroup(agregateGroup, c) == false) {
				logger.info("will insert " + c.getLastname());
				toInsert.add(c);
			} else {
				logger.info("skip " + c.getLastname());
			}
		}
		for (IAddressCollector collector: collectors) {
			collector.collectContacts(userId, userPassword, toInsert);
		}
	}

	private boolean checkContactExistsInGroup(ContactGroup agregateGroup, MinigContact c) {
		for (MinigEmail email: c.getEmails().values()) {
			if (!agregateGroup.find(email.getEmail(), 1).isEmpty()) {
				return true;
			}
		}
		return false;
	}

}
