/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.book.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import fr.aliasource.webmail.book.BookManager;
import fr.aliasource.webmail.book.ContactGroup;
import fr.aliasource.webmail.book.IBookSource;
import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigContact;
import fr.aliasource.webmail.proxy.ProxyConfiguration;

public class AllContacts implements IBookSource {

	private class AgregateContactGroup implements ContactGroup {

		private final String userId;
		private final String userPassword;

		public AgregateContactGroup(String userId, String userPassword) {
			this.userId = userId;
			this.userPassword = userPassword;
		}

		@Override
		public List<IMinigContact> findAll() {
			return AllContacts.this.findAll(userId, userPassword);
		}

		@Override
		public List<IMinigContact> find(String query, int limit) {
			return AllContacts.this.find(userId, userPassword, query, limit);
		}

		@Override
		public int count() {
			return AllContacts.this.count(userId, userPassword);
		}

		@Override
		public void insert(List<MinigContact> c) {
		}

		@Override
		public GroupType getType() {
			return GroupType.READ_ONLY;
		}

		@Override
		public Id getId() {
			return BookManager.ALL_SOURCE_ID;
		}

		@Override
		public String getDisplayName() {
			return "all";
		}
		
	}
	
	private Set<IBookSource> otherSources;
	private Log logger = LogFactory.getLog(getClass());

	public AllContacts(Collection<IBookSource> allSources) {
		this.otherSources = new HashSet<IBookSource>();
		otherSources.addAll(allSources);
		logger.info("AllContacts virtual source created  for "+otherSources.size() +" sources");
	}

	@Override
	public void init(ProxyConfiguration pc) {
		if (logger.isDebugEnabled()) {
			logger.debug("init");
		}
	}

	@Override
	public void shutdown() {
		if (logger.isDebugEnabled()) {
			logger.debug("shutdown");
		}
	}

	private List<IMinigContact> findAll(String userId, String userPassword) {
		Set<IMinigContact> ret = Sets.newTreeSet();
		for (IBookSource bs : otherSources) {
			for (ContactGroup group: bs.listGroups(userId, userPassword)) {
				ret.addAll(group.findAll());
			}
		}
		return Lists.newArrayList(ret);
	}

	private List<IMinigContact> find(String userId, String userPassword, String query, int limit) {
		List<IMinigContact> ret = Lists.newArrayList();
		for (IBookSource bs : otherSources) {
			for (ContactGroup group: bs.listGroups(userId, userPassword)) {
				ret.addAll(group.find(query, limit));
			}
		}
		Collections.sort(ret);
		return ret;
	}

	private int count(String userId, String userPassword) {
		int count = 0;
		for (IBookSource bs : otherSources) {
			for (ContactGroup group: bs.listGroups(userId, userPassword)) {
				count += group.count();
			}
		}
		return count;
	}

	@Override
	public List<? extends ContactGroup> listGroups(String userId, String userPassword) {
		return Lists.newArrayList(new AgregateContactGroup(userId, userPassword));
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((otherSources == null) ? 0 : otherSources.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AllContacts other = (AllContacts) obj;
		if (otherSources == null) {
			if (other.otherSources != null)
				return false;
		} else if (!otherSources.equals(other.otherSources))
			return false;
		return true;
	}
	
}
