package fr.aliasource.webmail.common.message;

import static org.minig.imap.MimeMessageFactory.createSimpleMimePart;
import static org.minig.imap.MimeMessageFactory.createSimpleMimeTree;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.easymock.EasyMock;
import org.junit.Assert;
import org.junit.Test;
import org.minig.imap.IMAPHeaders;
import org.minig.imap.MimeMessageTestUtils;
import org.minig.imap.mime.IMimePart;
import org.minig.imap.mime.MimeAddress;
import org.minig.imap.mime.MimeMessage;
import org.minig.imap.mime.MimePart;
import org.obm.mail.conversation.MailMessage;
import org.obm.mail.imap.StoreException;
import org.obm.mail.message.MailMessageAttachment;
import org.obm.mail.message.MessageFetcher;
import org.obm.mail.message.MessageLoader;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Iterables;


public class MessageLoaderTest {
	
	private static final String FAKE_STREAM_CONTENT = "fake stream content";
	
	private ByteArrayInputStream fakeStream() {
		return stringAsContent(FAKE_STREAM_CONTENT);
	}

	private ByteArrayInputStream stringAsContent(String content) {
		return new ByteArrayInputStream(content.getBytes());
	}
	

	private MimePart createTextHtmlPart() {
		return createSimpleMimePart("TEXT", "HTML", null, "7BIT", ImmutableMap.of("CHARSET", "US-ASCII"));
	}

	private MimePart createTextPlainPart() {
		return createSimpleMimePart("TEXT", "PLAIN", null, "7BIT", ImmutableMap.of("CHARSET", "US-ASCII"));
	}
	
	private MimePart createAttachment(String filename) {
		return createSimpleMimePart("APPLICATION", "OCTET-STREAM", null, "BASE64", ImmutableMap.of("NAME", filename));
	}

	private MimePart createApplicationPart(String mimeSubType, String filename) {
		return createSimpleMimePart("APPLICATION", mimeSubType, null, "BASE64", ImmutableMap.of("NAME", filename));
	}

	
	private MimeMessage createMimeMessage(String mimeType, String mimeSubtype, MimePart... parts) {
		return createSimpleMimeTree(mimeType, mimeSubtype, null, null, new HashMap<String, String>(), parts);
	}
	
	private MimePart createMultipartPart(String mimeType, String mimeSubtype, MimePart... parts) {
		return createSimpleMimePart(mimeType, mimeSubtype, null, null, new HashMap<String, String>(), parts);
	}
	
	private MimePart createNestedPart(String multipartSubtype, MimePart... parts) {
		MimePart message = createSimpleMimePart("message", "rfc822", null, "7BIT", new HashMap<String, String>(), parts);
		message.setMultipartSubtype(multipartSubtype);
		return message;
	}
	
	private IMimePart getPart(MimeMessage simpleMessage, String addr) {
		return MimeMessageTestUtils.getPartByAddress(simpleMessage, new MimeAddress(addr));
	}
	
	@Test
	public void testSimplestMail() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createSimpleMimeTree("TEXT", "PLAIN", null, "7BIT", ImmutableMap.of("CHARSET", "US-ASCII"));
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, simpleMessage.getMimePart())).andReturn(fakeStream());
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getPlainSubtypePriority(), true, simpleMessage);
		MailMessage message = messageLoader.fetch();
		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/plain"), message.getBody().availableFormats());
		Assert.assertEquals(FAKE_STREAM_CONTENT, message.getBody().getValue("text/plain"));
	}

	@Test
	public void testMailWithPlainAlternative() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "alternative", 
				createTextPlainPart(),
				createTextHtmlPart());
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, simpleMessage.getChildren().get(0))).andReturn(stringAsContent("text content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getPlainSubtypePriority(), true, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/plain"), message.getBody().availableFormats());
		Assert.assertEquals("text content", message.getBody().getValue("text/plain"));
	}

	@Test
	public void testMailWithHtmlAlternative() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "alternative", 
				createTextPlainPart(), createTextHtmlPart());
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, simpleMessage.getChildren().get(1))).andReturn(stringAsContent("html content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getHtmlSubtypePriority(), true, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/html"), message.getBody().availableFormats());
		Assert.assertEquals("html content", message.getBody().getValue("text/html"));
	}

	@Test
	public void testMailWithHtmlAlternative1() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "mixed",
				createMultipartPart("multipart", "alternative",
						createTextPlainPart(), 
						createMultipartPart("multipart", "related", 
								createTextHtmlPart(), 
								createAttachment("JPEG FILE"))),
						createApplicationPart("vnd.oasis.opendocument.spreadsheet", "Fichier calc vide.ods"));
		
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "1.2.1"))).andReturn(stringAsContent("TEXT HTML CONTENT"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		final MessageLoader messageLoader = new MessageLoader(messageFetcher, getHtmlSubtypePriority(), false, simpleMessage);
		MailMessage message = messageLoader.fetch();
		
		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/html"), message.getBody().availableFormats());
		Assert.assertEquals("TEXT HTML CONTENT", message.getBody().getValue("text/html"));
	}
	
	@Test
	public void testMailMixedPlusAlternative() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "mixed",
				createMultipartPart("multipart", "alternative",
					createTextPlainPart(),
					createTextHtmlPart()),
				createTextPlainPart());
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "1.1"))).andReturn(stringAsContent("plain content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getPlainSubtypePriority(), true, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/plain"), message.getBody().availableFormats());
		Assert.assertEquals("plain content", message.getBody().getValue("text/plain"));
	}

	@Test
	public void testMailMixedPlusAlternativePickHtml() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "mixed",
				createMultipartPart("multipart", "alternative",
					createTextPlainPart(),
					createTextHtmlPart()),
				createTextPlainPart());
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "1.2"))).andReturn(stringAsContent("html content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getHtmlSubtypePriority(), true, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/html"), message.getBody().availableFormats());
		Assert.assertEquals("html content", message.getBody().getValue("text/html"));
	}

	@Test
	public void testMailAlternativePlusMixed() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "alternative",
				createTextPlainPart(),
				createMultipartPart("multipart", "mixed",
					createTextHtmlPart()));
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "2.1"))).andReturn(stringAsContent("html content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getHtmlSubtypePriority(), true, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/html"), message.getBody().availableFormats());
		Assert.assertEquals("html content", message.getBody().getValue("text/html"));
	}
	
	@Test
	public void testForwardedMail() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "mixed",
				createTextPlainPart(),
				createNestedPart("alternative", createTextPlainPart(), createTextHtmlPart()));
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "1"))).andReturn(stringAsContent("text content"));
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "2.2"))).andReturn(stringAsContent("html forwarded content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, getPart(simpleMessage, "2"))).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getHtmlSubtypePriority(), false, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/plain"), message.getBody().availableFormats());
		Assert.assertEquals("text content", message.getBody().getValue("text/plain"));
		MailMessage forwardedMessage = Iterables.getFirst(message.getForwardMessage(), null);
		Assert.assertNotNull(forwardedMessage);
		Assert.assertEquals(ImmutableSet.of("text/html"), forwardedMessage.getBody().availableFormats());
		Assert.assertEquals("html forwarded content", forwardedMessage.getBody().getValue("text/html"));
	}
	
	@Test
	public void testAttachementInAlternativeNested() throws IOException, StoreException {
		MessageFetcher messageFetcher = EasyMock.createMock(MessageFetcher.class);
		MimeMessage simpleMessage = createMimeMessage("multipart", "mixed",
				createTextPlainPart(),
				createNestedPart("mixed", 
					createMultipartPart("multipart", "alternative",
							createTextPlainPart(), createTextHtmlPart()),
					createAttachment("myFile")));
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "1"))).andReturn(stringAsContent("text content"));
		EasyMock.expect(messageFetcher.fetchPart(simpleMessage, getPart(simpleMessage, "2.1.2"))).andReturn(stringAsContent("html forwarded content"));
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, simpleMessage)).andReturn(new IMAPHeaders());
		EasyMock.expect(messageFetcher.fetchPartHeaders(simpleMessage, getPart(simpleMessage, "2"))).andReturn(new IMAPHeaders());
		EasyMock.replay(messageFetcher);
		
		MessageLoader messageLoader = new MessageLoader(messageFetcher, getHtmlSubtypePriority(), false, simpleMessage);
		MailMessage message = messageLoader.fetch();

		EasyMock.verify(messageFetcher);
		Assert.assertNotNull(message);
		Assert.assertEquals(ImmutableSet.of("text/plain"), message.getBody().availableFormats());
		Assert.assertEquals("text content", message.getBody().getValue("text/plain"));
		MailMessage forwardedMessage = Iterables.getFirst(message.getForwardMessage(), null);
		Assert.assertNotNull(forwardedMessage);
		Assert.assertEquals(ImmutableSet.of("text/html"), forwardedMessage.getBody().availableFormats());
		Assert.assertEquals("html forwarded content", forwardedMessage.getBody().getValue("text/html"));
		MailMessageAttachment att = forwardedMessage.getAttachments().get(0);
		Assert.assertEquals("myFile", att.getDisplayName());
		Assert.assertEquals(new MimeAddress("2.2"), att.getPart().getAddress());
	}

	private List<String> getPlainSubtypePriority() {
		final String[] plainMimeSubtypePriority = new String[]{"plain", "html"};
		return Arrays.asList(plainMimeSubtypePriority);
	}
	
	private List<String> getHtmlSubtypePriority() {
		final String[] htmlMimeSubtypePriority = new String[]{"html", "plain"};
		return Arrays.asList(htmlMimeSubtypePriority);
	}
	
}
