/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.common.message;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.minig.imap.mime.IMimePart;
import org.minig.imap.mime.MimeMessage;
import org.obm.mail.conversation.MailMessage;
import org.obm.mail.conversation.MessageId;
import org.obm.mail.imap.StoreException;
import org.obm.mail.message.MailMessageAttachment;
import org.obm.mail.message.MessageFetcher;

import com.google.common.collect.Iterables;
import com.google.common.io.Files;
import com.google.common.io.InputSupplier;

import fr.aliasource.webmail.common.Activator;
import fr.aliasource.webmail.common.IAccount;
import fr.aliasource.webmail.common.folders.IFolder;
import fr.aliasource.webmail.common.imap.IStoreConnection;

/**
 * Creates a {@link MailMessage} from a {@link MessageId}.
 */
public class MailMessageLoader {

	// private static final String[] HEADS_LOAD = new String[] { "Subject",
	// "From", "Date", "To", "Cc", "Bcc", "X-Mailer", "User-Agent",
	// "Message-ID" };

	private static final Log logger = LogFactory.getLog(MailMessageLoader.class);
	
	private IFolder f;
	private AttachmentManager atMgr;
	private final List<String> htmlMimeSubtypePriority;
		
	/**
	 * Creates a message loader with the given {@link AttachmentManager} for the
	 * given {@link IFolder}
	 */
	public MailMessageLoader(AttachmentManager atMgr, IFolder f) {
		this.f = f;
		this.atMgr = atMgr;
		this.htmlMimeSubtypePriority = Arrays.asList("html", "plain");
	}

	public void select(IStoreConnection protocol) throws IOException, StoreException {
		protocol.select(f.getName());
	}

	public MailMessage fetch(IAccount account, MessageId mid, IStoreConnection protocol, boolean bodyOnly)
		throws IOException, StoreException {
		Collection<MimeMessage> mts = protocol.uidFetchBodystructure(Arrays.asList(mid.getImapId()));
		MimeMessage tree = Iterables.getFirst(mts, null);
		if (tree != null) {
			MessageFetcher messageFetcher = new MessageFetcherImpl(protocol);
			MessageLoader helper = new MessageLoader(messageFetcher, htmlMimeSubtypePriority, bodyOnly, tree);
			helper.setHeadersFilters(Activator.getDefault().getMailHeadersFilters(), account);
			MailMessage message = helper.fetch();
			if (!bodyOnly) {
				registerAttachments(message, messageFetcher);
			}
			return message;
		}
		return null;
	}

	private void registerAttachments(MailMessage message, MessageFetcher messageFetcher) {
		for (MailMessageAttachment attachment: message.getAttachments()) {
			registerAttachment(message, attachment, messageFetcher);
		}
		for (MailMessage nested: message.getForwardMessage()) {
			registerAttachments(nested, messageFetcher);
		}
	}

	private void registerAttachment(MailMessage message, MailMessageAttachment attachment, MessageFetcher messageFetcher) {
		File at = atMgr.open(attachment.getAtMgrId());
		if (!at.exists()) {
			IMimePart part = attachment.getPart();
			try {
				final InputStream stream = messageFetcher.fetchPart(message, part);
				Files.copy(new InputSupplier<InputStream>() {
					@Override
					public InputStream getInput() throws IOException {
						return stream;
					}
				}, at);
				storeMetadata(attachment, at);
			} catch (Exception e) {
				logger.error("Error storing metadata for " + message.getUid(), e);
			}
		}
	}

	private void storeMetadata(MailMessageAttachment attachment, File out) 
		throws FileNotFoundException, ParserConfigurationException, FactoryConfigurationError, TransformerException {
			atMgr.storeMetadata(attachment.getAtMgrId(), attachment.getPart(), out.length());
	}
	
}
