/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.ldap;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.naming.NamingException;
import javax.naming.directory.DirContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.collect.Lists;

import fr.aliasource.utils.LdapUtils;
import fr.aliasource.webmail.book.AbstractContactGroup;
import fr.aliasource.webmail.book.AbstractContactGroup.Origin;
import fr.aliasource.webmail.book.IBookSource;
import fr.aliasource.webmail.book.IMinigContact;
import fr.aliasource.webmail.book.MinigContact;
import fr.aliasource.webmail.book.MinigEmail;
import fr.aliasource.webmail.proxy.ProxyConfiguration;

public class BookSource implements IBookSource {

	private static final Origin origin = new Origin("obm_ldap");

	private class ContactGroup extends AbstractContactGroup {

		private final String userId;
		private final String userPassword;

		public ContactGroup(String userId, String userPassword) {
			super(new Id("obm_ldap_all", origin), "obm_ldap_all");
			this.userId = userId;
			this.userPassword = userPassword;
		}

		@Override
		public List<IMinigContact> findAll() {
			return BookSource.this.findAll(userId, userPassword);
		}

		@Override
		public List<IMinigContact> find(String query, int limit) {
			return BookSource.this.find(userId, userPassword, query, limit);
		}

		@Override
		public int count() {
			return BookSource.this.count(userId, userPassword);
		}

		@Override
		public void insert(List<MinigContact> c) {
		}

		@Override
		public GroupType getType() {
			return GroupType.READ_ONLY;
		}

		@Override
		public IBookSource getSource() {
			return BookSource.this;
		}
		
	}
	
	private Configuration conf;
	private Log logger;

	public BookSource() {
		this.logger = LogFactory.getLog(getClass());
	}

	@Override
	public List<? extends ContactGroup> listGroups(String userId, String userPassword) {
		return Lists.newArrayList(new ContactGroup(userId, userPassword));
	}
	
	private int count(String userId, String userPassword) {
		return findAll(userId, userPassword).size();
	}

	private List<IMinigContact> findAll(String userId, String userPassword) {
		return find(userId, userPassword, "", Integer.MAX_VALUE);
	}

	private List<IMinigContact> find(String userId, String userPassword,
			String query, int limit) {
		DirContext ctx = null;
		List<IMinigContact> ret = new LinkedList<IMinigContact>();
		String domain = "";
		int idx = userId.indexOf("@");
		if (idx > 0) {
			domain = userId.substring(idx + 1);
		}
		try {
			ctx = conf.getConnection();

			LdapUtils u = new LdapUtils(ctx, conf.getBaseDn().replace("%d",
					domain));
			List<Map<String, List<String>>> l = u.getAttributes(conf
					.getFilter(), query, new String[] { "cn", "sn",
					"givenName", "mail" });
			l = l.subList(0, Math.min(limit, l.size()));
			for (Map<String, List<String>> m : l) {
				String sn = uniqueAttribute("sn", m);
				String givenName = uniqueAttribute("givenName", m);
				String cn = uniqueAttribute("cn", m);
				if (sn.length() == 0 || givenName.length() == 0) {
					sn = cn;
					givenName = "";
				}
				MinigContact c = new MinigContact(sn, givenName, "", "", "",
						"", "", "", "");
				List<String> mails = m.get("mail");
				String lblRoot = "INTERNET;X-OBM-Ref";
				int i = 1;
				for (String mail : mails) {
					c.addEmail(lblRoot + (i++), new MinigEmail(mail));
				}

				ret.add(c);
			}
		} catch (NamingException e) {
			logger.error("findAll error", e);
		} finally {
			conf.cleanup(ctx);
		}
		return ret;
	}

	private String uniqueAttribute(String string, Map<String, List<String>> m) {
		List<String> cnl = m.get(string);
		if (cnl == null || cnl.size() == 0) {
			return "";
		} else {
			return cnl.get(0);
		}
	}

	@Override
	public void init(ProxyConfiguration pc) {
		this.conf = new Configuration(pc);
		logger.info("ldap book source initialised.");
	}

	@Override
	public void shutdown() {
	}

}
