/*
 *  BEGIN LICENSE BLOCK Version: GPL 2.0
 * 
 * The contents of this file are subject to the GNU General Public License
 * Version 2 or later (the "GPL").
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * 
 * The Initial Developer of the Original Code is MiniG.org project members
 * 
 * END LICENSE BLOCK
 */

package fr.aliasource.webmail.client.addressbook;

import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Widget;

import fr.aliasource.webmail.client.I18N;
import fr.aliasource.webmail.client.View;
import fr.aliasource.webmail.client.addressbook.ContactGroups.ContactGroupsDelegate;
import fr.aliasource.webmail.client.addressbook.events.CreateMailingListEvent;
import fr.aliasource.webmail.client.addressbook.events.MailingListEventsHandler;
import fr.aliasource.webmail.client.addressbook.events.RemoveMailingListEvent;
import fr.aliasource.webmail.client.addressbook.events.UpdateMailingListEvent;
import fr.aliasource.webmail.client.ctrl.AjaxCall;
import fr.aliasource.webmail.client.shared.ContactGroup;
import fr.aliasource.webmail.client.shared.ContactGroup.DefaultGroup;
import fr.aliasource.webmail.client.shared.book.UIMailingList;
import fr.aliasource.webmail.client.shared.book.UiContact;
import fr.aliasource.webmail.client.widget.MinigTextBox;

/**
 * Address book widget
 * 
 * @author tom
 * 
 */
public class AddressBook extends Composite implements MailingListEventsHandler, ContactGroupsDelegate {

	@UiField
	MinigTextBox searchField;
	@UiField
	ScrollPanel groupsListSp;
	@UiField
	ScrollPanel contactListSp;
	@UiField
	ScrollPanel contactDisplaySp;
	@UiField
	HorizontalPanel hp;
	
	ContactGroup current;
	

	interface Binder extends UiBinder<Widget, AddressBook> {
	}

	private static final Binder BINDER = GWT.create(Binder.class);
	protected static final String MAILING_LISTS = "mailingLists";

	private View webmail;

	private ContactGroups groupsList;
	private ContactList contactList;
	private ContactDisplay display;

	public AddressBook(View wm) {
		initWidget(BINDER.createAndBindUi(this));
		this.webmail = wm;

		initSearchField();

		
    	groupsList = new ContactGroups(webmail);
		groupsListSp.add(groupsList);
    	contactList = new ContactList(webmail);
    	
    	groupsList.setDelegate(this);
    
		contactListSp.add(contactList);

		display = new ContactDisplay(webmail);
		display.addMailingListEventsHandler(this);
		contactList.setDelegate(display);
		contactDisplaySp.add(display);
		
		resize( Window.getClientHeight());
		
		
		hp.setCellWidth(contactDisplaySp, "100%");

		addTabListener();
	

		Window.addResizeHandler(new ResizeHandler() {
			@Override
			public void onResize(ResizeEvent event) {
				resize(event.getHeight());
			}
		});
	}

	

	private void initSearchField() {
		searchField.setDescription(I18N.strings.searchContactField());
		searchField.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent ev) {
				if (!searchField.getValue().isEmpty()) {
					searchResult();
				} else {
					reset();
				}
			}
		});
	}

	private void searchResult() {
		webmail.getSpinner().startSpinning();
		AsyncCallback<UiContact[]> ac = new AsyncCallback<UiContact[]>() {
			public void onFailure(Throwable caught) {
				webmail.log("loadGroup failure");
				webmail.getSpinner().stopSpinning();
			}

			public void onSuccess(UiContact[] contacts) {
				contactList.clear();
				display.reset();

				int row = groupsList.getRowCount() - 1;
				if (groupsList.getSelected() != -1) {
					groupsList.removeSelected(groupsList.getSelected());
				}
				groupsList.getRowFormatter().setVisible(row, true);
				groupsList.setSelected(row);
				groupsList.setWidget(row, 1,
						new Label(Integer.toString(contacts.length), false));

				contactList.setDisplayAllContact(false);
				contactList.updateContacts(contacts);

				webmail.getSpinner().stopSpinning();
			}

		};
		AjaxCall.contacts.getContacts(groupsList.getAllGroup(),
				searchField.getValue(), ac);
	}

	private void reset() {
		contactList.reset();
		groupsList.removeSelected(groupsList.getSelected());
		loadGroups();
		display.reset();
	}

	private void addTabListener() {
		webmail.getTabPanel().addSelectionHandler(
				new SelectionHandler<Integer>() {
					@Override
					public void onSelection(SelectionEvent<Integer> event) {
						if (event.getSelectedItem() == View.ADDRESSBOOK) {
							loadBook();
						}
					}
				});
	}

	protected void loadBook() {
		loadGroups();
	}

	private void loadGroups() {
		if (groupsList == null) {
			return;
		}
		webmail.getSpinner().startSpinning();
		AsyncCallback<ContactGroup[]> ac = new ContactGroupAsyncCallback() {
			@Override
			public void onFailure(Throwable caught) {
				webmail.log("contactGroups failure");
				webmail.getSpinner().stopSpinning();
			}

			@Override
			public void onSuccess(Set<ContactGroup> groups) {
				webmail.getSpinner().stopSpinning();
				groupsList.updateGrid(groups);
			}

		};
		
		AjaxCall.contacts.getContactGroups(ac);
		
		
	}

	public void resize(int height) {
		groupsListSp.setHeight(height - 150 + "px");
		contactDisplaySp.setHeight(height - 150 + "px");
		contactListSp.setHeight(height - 150 + "px");
	}

	public ContactList getContactList() {
		return contactList;
	}

	public ContactDisplay getContactDisplay() {
		return display;
	}

	public void clearSearchField() {
		searchField.clear();
	}
	

	@Override
	public void onRemoveMailingList(RemoveMailingListEvent event) {
		loadGroups();
		groupSelected(current);
	}

	@Override
	public void onCreateMailingList(CreateMailingListEvent event) {
		loadGroups();
		groupSelected(current);
	}

	@Override
	public void onUpdateMailingList(UpdateMailingListEvent event) {
		groupSelected(current);
		
	}

	//TODO can be smaller
	@Override
	public void groupSelected(ContactGroup group) {
		webmail.log("Group selected " + group.getDisplayName(), null);
		contactList.setDisplayAllContact(false);
		webmail.getSpinner().startSpinning();
		current = group;
		if(!DefaultGroup.MAILING.getGroup().equals(group)){
			AjaxCall.contacts.getContacts(group, "",  new AsyncCallback<UiContact[]>() {
				public void onFailure(Throwable caught) {
					webmail.log("loadGroup failure");
					webmail.getSpinner().stopSpinning();
				}
	
				public void onSuccess(UiContact[] contacts) {
					webmail.getSpinner().stopSpinning();
					contactList.updateContacts(contacts);
				}
	
			});
		} else {
			AjaxCall.mailingList.listAllMailingList(new AsyncCallback<UIMailingList[]>() {
				
				@Override
				public void onSuccess(UIMailingList[] contacts) {
					webmail.getSpinner().stopSpinning();
					contactList.updateContacts(contacts);
					
				}
				
				@Override
				public void onFailure(Throwable arg0) {
					webmail.log("loadMailingsLists failure");
					webmail.getSpinner().stopSpinning();
					
				}
			});
		}
	}
}
