/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.client.addressbook;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import fr.aliasource.webmail.client.I18N;
import fr.aliasource.webmail.client.View;
import fr.aliasource.webmail.client.addressbook.ContactCardDisplay.ContactCardDisplayDelegate;
import fr.aliasource.webmail.client.addressbook.ContactList.ContactListDelegate;
import fr.aliasource.webmail.client.addressbook.MailingListDisplay.MailingListDisplayDelegate;
import fr.aliasource.webmail.client.addressbook.MultiContactDisplay.MultiContactDisplayDelegate;
import fr.aliasource.webmail.client.addressbook.events.AbstractMailingListEvent;
import fr.aliasource.webmail.client.addressbook.events.CreateMailingListEvent;
import fr.aliasource.webmail.client.addressbook.events.HasMailingListEventsHandlers;
import fr.aliasource.webmail.client.addressbook.events.MailingListEventsHandler;
import fr.aliasource.webmail.client.addressbook.events.RemoveMailingListEvent;
import fr.aliasource.webmail.client.addressbook.events.UpdateMailingListEvent;
import fr.aliasource.webmail.client.ctrl.AjaxCall;
import fr.aliasource.webmail.client.shared.book.AbstractUIContact;
import fr.aliasource.webmail.client.shared.book.UIMailingList;
import fr.aliasource.webmail.client.shared.book.UiContact;
import fr.aliasource.webmail.client.shared.mailing.MailingListsUtils;

/**
 * Displays informations about selected contacts
 * 
 * @author matthieu
 * 
 */
public class ContactDisplay extends VerticalPanel  implements HasMailingListEventsHandlers, 
	ContactListDelegate, MailingListDisplayDelegate, MultiContactDisplayDelegate, ContactCardDisplayDelegate {

	private Set<UiContact> contacts;
	private View ui;

	private MailingListDisplay mailingListDisplay= GWT.create(MailingListDisplay.class);
	
	private HTML empty;

	public ContactDisplay(View ui) {
		super();
		this.ui = ui;
		setWidth("100%");
		empty = new HTML(I18N.strings.emptyContactDisplay());
		add(empty);
		mailingListDisplay.setDelegate(this,ui);
		contacts = new HashSet<UiContact>();
	}

	public void reset() {
		clear();
		add(empty);
		contacts.clear();
		mailingListDisplay.reset();
	}

	

	@Override
	public void selectContact(AbstractUIContact co) {
		if(co!=null && co instanceof UiContact){
			contacts.add((UiContact)co);
		}
		
		doDisplay(co);
	}

	@Override
	public void unSelectContact(AbstractUIContact co) {
		if(co!=null && co instanceof UiContact){
			contacts.remove((UiContact)co);
		}
		doDisplay(co);
	}

	private void doDisplay(AbstractUIContact co) {
		clear();
		if(co!=null && co instanceof UiContact){
			if (contacts.size() > 1) {
				Widget w = new MultiContactDisplay(contacts, ui,this);
				add(w);
			} else if (contacts.size() == 1) {
				ContactCardDisplay w = new ContactCardDisplay(contacts.iterator().next(), ui);
				w.setDelegate(this);
				add(w);
			} else {
				add(empty);
			}
		} else {
			mailingListDisplay.setData((UIMailingList)co);
			mailingListDisplay.setCreating(false);
			add(mailingListDisplay);
		}
		
	}

	@Override
	public void createMailing() {
		clear();
		mailingListDisplay.reset();
		mailingListDisplay.setCreating(true);
		add(mailingListDisplay);
	}


	@Override
	public void createMailing(Set<UiContact> contacts) {
		clear();
		mailingListDisplay.reset();
		mailingListDisplay.setCreating(true);
		mailingListDisplay.setData(contacts);
		add(mailingListDisplay);
	}

	
	@Override
	public void save(UIMailingList mailingList) {
		if(isValid(mailingList)){
			ui.getSpinner().startSpinning();
			final boolean create = mailingList.getId()==null;
			AsyncCallback<UIMailingList> ac = 	new AsyncCallback<UIMailingList>() {
				
				@Override
				public void onSuccess(UIMailingList ret) {
					ui.getSpinner().stopSpinning();
					mailingListDisplay.reset();
					mailingListDisplay.setData(ret);
					mailingListDisplay.setCreating(false);
					mailingListDisplay.showMessages(getCreateOrUpdateMessages(ret));
					if(create){
						fireEvent(new CreateMailingListEvent(ret));
					} else {
						fireEvent(new UpdateMailingListEvent(ret));
					}
				
					
				}
				
				private List<String> getCreateOrUpdateMessages(
						UIMailingList mailingList) {
					List<String> messages = new ArrayList<String>();
					if(create){
						messages.add(I18N.strings.mailingCreateOK());
					} else {
						messages.add(I18N.strings.mailingUpdateOK());
					}
					return messages;
				}
				
				@Override
				public void onFailure(Throwable arg0) {
					ui.log("loadGroup failure");
					ui.getSpinner().stopSpinning();
				}
			};
			
			if(create){
				AjaxCall.mailingList.createMailingList(mailingList, ac);
			} else {
				AjaxCall.mailingList.modifyMailingList(mailingList, ac);
			}
		}
	}


	private boolean isValid(UIMailingList mailingList) {
		List<String> errors = new ArrayList<String>();
		if(mailingList.getDisplayName()==null || mailingList.getDisplayName().isEmpty()){
			errors.add(I18N.strings.errorEmptyField(I18N.strings.mailingName()));
		}
		if(mailingList.getEmails()==null || mailingList.getEmails().isEmpty()){
			errors.add(I18N.strings.errorEmptyField(I18N.strings.participants()));
		}
		
		if(!errors.isEmpty()){
			mailingListDisplay.showErrors(errors);
			return false;
		}
		return true;
	}

	@Override
	public void delete(final UIMailingList mailing) {
		ui.getSpinner().startSpinning();
		AjaxCall.mailingList.removeMailingList(mailing.getId(), new AsyncCallback<Void>() {
			
			@Override
			public void onSuccess(Void arg0) {
				ui.getSpinner().stopSpinning();
				mailingListDisplay.reset();
				mailingListDisplay.showMessages(getDeleteMessages());
				mailingListDisplay.setCreating(true);
				fireEvent(new RemoveMailingListEvent(mailing));
			}
			
			private List<String> getDeleteMessages() {
				List<String> messages = new ArrayList<String>();
				
				messages.add(I18N.strings.mailingDeleteOK());
			
				return messages;
			}

			@Override
			public void onFailure(Throwable arg0) {
				ui.log("loadMailing failure");
				ui.getSpinner().stopSpinning();
				
			}
		});
		
	}

	@Override
	public void cancel() {
		reset();
	}

	@Override
	public void edit(UiContact c) {
		ui.getSpinner().startSpinning();
		AjaxCall.mailingList.getMailingListFromId(MailingListsUtils.getId(c.getLastname()), new AsyncCallback<UIMailingList>() {
			
			@Override
			public void onSuccess(UIMailingList mailing) {
				ui.getSpinner().stopSpinning();
				selectContact(mailing);
			}
			
			@Override
			public void onFailure(Throwable arg0) {
				ui.log("loadMailing failure");
				ui.getSpinner().stopSpinning();
				
			}
		});

	}
	

	@Override
	public void addMailingListEventsHandler(MailingListEventsHandler handler) {
		addHandler(handler, AbstractMailingListEvent.getType());	
	}
	

}
