/* ***** BEGIN LICENSE BLOCK *****
 * Version: GPL 2.0
 *
 * The contents of this file are subject to the GNU General Public
 * License Version 2 or later (the "GPL").
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *   MiniG.org project members
 *
 * ***** END LICENSE BLOCK ***** */

package fr.aliasource.webmail.client.addressbook;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.Label;

import fr.aliasource.webmail.client.I18N;
import fr.aliasource.webmail.client.View;
import fr.aliasource.webmail.client.shared.book.AbstractUIContact;
import fr.aliasource.webmail.client.shared.book.UIMailingList;

public class ContactList extends Grid {

	private View webmail;
	private boolean displayAllContact;
	private Map<CheckBox, Integer> checkedItems;
	private Map<AbstractUIContact, Integer> contactsMap = new HashMap<AbstractUIContact, Integer>();
	private ContactListDelegate delegate;
	
	
	interface ContactListDelegate {

		void selectContact(AbstractUIContact co);

		void unSelectContact(AbstractUIContact co);

		void createMailing();

		void reset();
		
	}
	

	/**
	 * @param delegate the delegate to set
	 */
	public void setDelegate(ContactListDelegate delegate) {
		this.delegate = delegate;
	}

	public ContactList(View webmail) {
		super(1, 2);
		this.webmail = webmail;
		this.displayAllContact = false;
		getCellFormatter().setWidth(0, 1, "100%");
		setWidget(0, 1, new Label(I18N.strings.emptyContactList(), true));
		setCellSpacing(0);
		setStyleName("addressBookList");
		checkedItems = new HashMap<CheckBox, Integer>();
	}

	public void reset() {
		clear();
		resizeRows(1);
		setDisplayAllContact(false);
		setWidget(0, 1, new Label(I18N.strings.emptyContactList(), true));
		getRowFormatter().setStyleName(0, ""); // Remove css (contentInfo &&
		// selectedContactGroup)
		setHeight("");
		checkedItems.clear();
	}

	public void updateContacts(final AbstractUIContact[] contacts) {
		Arrays.sort(contacts, new ContactComp());
		clear();
		contactsMap.clear();
		int limit = getNbDisplayItems();
		if (limit > contacts.length || isDisplayAllContact()) {
			limit = contacts.length;
			setDisplayAllContact(true);
		}
		
		resizeRows(limit + 1);
		int start = 0;
		// Header
		setWidget(start, 0, new Label(I18N.strings.contactName()));
		getRowFormatter().setStyleName(0, "addressBookHeader");

		
		if(contacts instanceof UIMailingList[]){
			start++;
			resizeRows(limit + 1 + start);
			Anchor hl = new Anchor(I18N.strings.addMailing());
			hl.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent sender) {
					delegate.createMailing();
				}
			});
			setWidget(start, 1, hl);
			getRowFormatter().setStyleName(1, "contentInfo");
		}
		
		for (int i = 0; i < limit ; i++) {
			addContact(i+1+start, contacts[i]);
		}
		
		if (!isDisplayAllContact()) {
			resizeRows(limit + 2 + start);
			Anchor hl = new Anchor(I18N.strings.displayAllXContacts(Integer
					.toString(contacts.length)));
			hl.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent sender) {
					setDisplayAllContact(!isDisplayAllContact());
					clear();
					updateContacts(contacts);
					getRowFormatter().removeStyleName(getNbDisplayItems(),
							"contentInfo");
					delegate.reset();
				}
			});
			setWidget(limit + 1 + start, 1, hl);
			getRowFormatter().setStyleName(limit + 1 + start, "contentInfo");
			setHeight("100%"); // "Display all" link on grid bottom
		} else {
			setHeight("");
		}
		
		
	}

	private void addContact(int row, AbstractUIContact co) {

			CheckBox cb = new CheckBox();
			Label l = new Label(co.getDisplayName(), true);
			l.addClickHandler(createLabelClickListener(cb, co, row));
			cb.addClickHandler(createCbClickListener(cb, row, co));
			setWidget(row, 0, cb);
			setWidget(row, 1, l);
			getRowFormatter().setStyleName(row, "addressBookItem");
			contactsMap.put(co, row);
			if(co instanceof UIMailingList){
				cb.setVisible(false);
			}

	}

	private void selectNone() {
		for (CheckBox c : checkedItems.keySet()) {
			c.setValue(false);
			getRowFormatter().removeStyleName(checkedItems.get(c),
					"selectedContactGroup");
		}
		checkedItems.clear();
	}

	private ClickHandler createLabelClickListener(final CheckBox c,
			final AbstractUIContact co, final int row) {
		return new ClickHandler() {
			public void onClick(ClickEvent sender) {

				webmail.getAddressBook().getContactDisplay().reset();
				selectNone();

				if (!c.getValue()) {
					getRowFormatter().addStyleName(row, "selectedContactGroup");
					delegate.selectContact(co);
					c.setValue(true);
					checkedItems.put(c, row);
				} else {
					getRowFormatter().removeStyleName(row,
							"selectedContactGroup");
					delegate.unSelectContact(co);
					c.setValue(false);
					checkedItems.remove(c);
				}
			}
		};
	}

	private ClickHandler createCbClickListener(final CheckBox c, final int row,
			final AbstractUIContact co) {
		return new ClickHandler() {
			public void onClick(ClickEvent sender) {
				if (!c.getValue()) {
					getRowFormatter().removeStyleName(row,
							"selectedContactGroup");
					delegate.unSelectContact(co);
					checkedItems.remove(c);
				} else {
					getRowFormatter().addStyleName(row, "selectedContactGroup");
					delegate.selectContact(co);
					checkedItems.put(c, row);
				}
			}
		};
	}



	private int getNbDisplayItems() {
		int height = this.getParent().getOffsetHeight();
		return (height / 23) - 1;
	}

	public void setDisplayAllContact(boolean displayAllContact) {
		this.displayAllContact = displayAllContact;
	}

	public boolean isDisplayAllContact() {
		return displayAllContact;
	}

	private class ContactComp implements Comparator<AbstractUIContact> {
		public int compare(AbstractUIContact c1, AbstractUIContact c2) {
			if (c1.getDisplayName().compareToIgnoreCase(c2.getDisplayName()) > 0) {
				return 1;
			}
			return -1;
		}
	}

	

}
